/*
Fast Gauss Bilateral Filtering C++ code 

 Shin Yoshizawa (shin@riken.jp)

Copyright:(c) Shin Yoshizawa, 2014.


 All right is reserved by Shin Yoshizawa.
This C++ code files are allowed for only primary user of research and educational purposes. Don't use secondary: copy, distribution, diversion, business purpose, and etc.. 

 In no event shall the author be liable to any party for direct, indirect, 
special, incidental, or consequential damage arising out of the use of this 
program and source files. 

 The C++ code and its simple manual are available at 
http://www.riken.jp/brict/Yoshizawa/Research/FBGFilter.html
*/

#include"gaussfgt4D.h"


class fastgb{
 public:
  double scale;
  int IRC;
  fastgb(int dIRC){
    scale=1.0;
    IRC=dIRC;
  }
  virtual ~fastgb(){
  
  }
 inline double gauss(double dx, double dy, double dsig) {
    double dist = dx-dy;
    return exp(-(dist*dist)*dsig);
  }

  
 
//Fast O(N) Methods 
  
  


  


  //3D via FGT 4D
  void FastBilateralFilter(Image3D *input,double *sigma,double error){
    int Num = input->sx*input->sy*input->sz;
    int dim=4;
    gaussfgt4D* mygauss = new gaussfgt4D();
    mygauss->allocate(Num);
    double *dI = new double[Num];
    double *result = new double[Num];
    int i,j,k,l;
    l=0;
    for(i=0;i<input->sz;i++)
      for(j=0;j<input->sy;j++)
	for(k=0;k<input->sx;k++){
	  dI[l]=input->img[i][j][k];
	  
	  mygauss->sgx[l]=((double)(k))*input->px;
	  mygauss->sgy[l]=((double)(j))*input->py;
	  mygauss->sgz[l]=((double)(i))*input->pz;
	  mygauss->sgw[l]=input->img[i][j][k];
	  l++;
	}
    
   
    mygauss->gaussth1(dI,sigma,Num,dim,result,error,IRC);
    
    l=0;
    for(i=0;i<input->sz;i++)
      for(j=0;j<input->sy;j++)
	for(k=0;k<input->sx;k++){
	  dI[l]=1.0;
	  input->img[i][j][k]=result[l];
	  l++;
	}
     
    mygauss->gaussth2(dI,sigma,Num,dim,result,error,IRC);
    
    
    l=0;
    for(i=0;i<input->sz;i++)
      for(j=0;j<input->sy;j++)
	for(k=0;k<input->sx;k++){
	  
	  double dval = result[l];if(dval<=0.0)dval=1.0;
	  input->img[i][j][k] /= dval;
	  l++;
	}
    
    
    delete [] dI;
    delete [] result;
    delete mygauss;
  }


 

};

#undef INF

